@@ -358,6 +358,49 @@ class Yaro(object):
             return util.FileWrapper(body)
 
 
+class OYaro(object):
+    """WSGI wrapper for something that takes and returns Request.
+    
+    For use with bound methods (things that need a reference to the
+    instance of which they are a member when called).
+    """
+
+    def __init__(self, app, extra_props=None):
+        """Take the thing to wrap."""
+        self.app = app
+        self.extra_props = extra_props
+
+    def __call__(self, instance, environ, start_response):
+        """Create Request, call thing, unwrap results and respond."""
+        req = Request(environ, start_response, self.extra_props)
+        body = self.app(instance, req)
+        req.save_to_environ()
+        if body is None:
+            body = req.res.body
+        if not req.start_response_called:
+            req.start_response(req.res.status, req.res._headers, req.exc_info)
+            req.start_response_called = True
+        if isinstance(body, str):
+            return [body]
+        elif isiterable(body):
+            return body
+        else:
+            return util.FileWrapper(body)
+
+
+def oYaro(extra_props=None):
+    """Decorate a bound method so that it calls an OYaro instance."""
+    def oy_decorator(fn):
+        oy = OYaro(fn, extra_props)
+        def newfn(self, environ, start_response):
+            return oy(self, environ, start_response)
+        newfn.__name__ = fn.__name__
+        newfn.__dict__ = fn.__dict__
+        newfn.__doc__ = fn.__doc__
+        return newfn
+    return oy_decorator
+
+
 def isiterable(it):
     """Return True if 'it' is iterable else return False."""
     try:
@@ -405,7 +448,12 @@ if __name__ == '__main__':
     def zoink(req):
         return req.stuff
 
+    class OYaroTestApp(object):
+        @oYaro()
+        def hi(self, req):
+            return "Hi, my name is %s" % self.__class__.__name__
+
     try:
-        make_server('localhost', 9999, Yaro(boing)).serve_forever()
+        make_server('localhost', 9999, OYaroTestApp().hi).serve_forever()
     except KeyboardInterrupt, ki:
         print 'I said "Good day!"'
